@@ -22,13 +22,14 @@ from jinja2 import (Environment, FileSystemLoader, PrefixLoader, ChoiceLoader,
 from pelican.contents import Article, Draft, Page, Static, is_valid_content
 from pelican.readers import Readers
 from pelican.utils import (copy, process_translations, mkdir_p, DateFormatter,
-                           FileStampDataCacher)
+                           FileStampDataCacher, python_2_unicode_compatible)
 from pelican import signals
 
 
 logger = logging.getLogger(__name__)
 
 
+@python_2_unicode_compatible
 class Generator(object):
     """Baseclass generator"""
 
@@ -67,7 +68,7 @@ class Generator(object):
             extensions=self.settings['JINJA_EXTENSIONS'],
         )
 
-        logger.debug('template list: {0}'.format(self.env.list_templates()))
+        logger.debug('Template list: %s', self.env.list_templates())
 
         # provide utils.strftime as a jinja filter
         self.env.filters.update({'strftime': DateFormatter()})
@@ -121,13 +122,21 @@ class Generator(object):
         """
         if isinstance(paths, six.string_types):
             paths = [paths] # backward compatibility for older generators
+
+        # group the exclude dir names by parent path, for use with os.walk()
+        exclusions_by_dirpath = {}
+        for e in exclude:
+            parent_path, subdir = os.path.split(os.path.join(self.path, e))
+            exclusions_by_dirpath.setdefault(parent_path, set()).add(subdir)
+
         files = []
         for path in paths:
-            root = os.path.join(self.path, path)
+            # careful: os.path.join() will add a slash when path == ''.
+            root = os.path.join(self.path, path) if path else self.path
 
             if os.path.isdir(root):
                 for dirpath, dirs, temp_files in os.walk(root, followlinks=True):
-                    for e in exclude:
+                    for e in exclusions_by_dirpath.get(dirpath, ()):
                         if e in dirs:
                             dirs.remove(e)
                     reldir = os.path.relpath(dirpath, self.path)
@@ -140,9 +149,27 @@ class Generator(object):
         return files
 
     def add_source_path(self, content):
+        """Record a source file path that a Generator found and processed.
+        Store a reference to its Content object, for url lookups later.
+        """
         location = content.get_relative_source_path()
         self.context['filenames'][location] = content
 
+    def _add_failed_source_path(self, path):
+        """Record a source file path that a Generator failed to process.
+        (For example, one that was missing mandatory metadata.)
+        The path argument is expected to be relative to self.path.
+        """
+        self.context['filenames'][os.path.normpath(path)] = None
+
+    def _is_potential_source_path(self, path):
+        """Return True if path was supposed to be used as a source file.
+        (This includes all source files that have been found by generators
+        before this method is called, even if they failed to process.)
+        The path argument is expected to be relative to self.path.
+        """
+        return os.path.normpath(path) in self.context['filenames']
+
     def _update_context(self, items):
         """Update the context with the given items from the currrent
         processor.
@@ -153,6 +180,10 @@ class Generator(object):
                 value = list(value.items())  # py3k safeguard for iterators
             self.context[item] = value
 
+    def __str__(self):
+        # return the name of the class for logging purposes
+        return self.__class__.__name__
+
 
 class CachingGenerator(Generator, FileStampDataCacher):
     '''Subclass of Generator and FileStampDataCacher classes
@@ -470,10 +501,13 @@ class ArticlesGenerator(CachingGenerator):
                         context_signal=signals.article_generator_context,
                         context_sender=self)
                 except Exception as e:
-                    logger.warning('Could not process {}\n{}'.format(f, e))
+                    logger.error('Could not process %s\n%s', f, e,
+                        exc_info=self.settings.get('DEBUG', False))
+                    self._add_failed_source_path(f)
                     continue
 
                 if not is_valid_content(article, f):
+                    self._add_failed_source_path(f)
                     continue
 
                 self.cache_data(f, article)
@@ -492,11 +526,11 @@ class ArticlesGenerator(CachingGenerator):
                     context_sender=self)
                 all_drafts.append(draft)
             else:
-                logger.warning("Unknown status %s for file %s, skipping it." %
-                               (repr(article.status),
-                                repr(f)))
+                logger.error("Unknown status '%s' for file %s, skipping it.",
+                               article.status, f)
 
-        self.articles, self.translations = process_translations(all_articles)
+        self.articles, self.translations = process_translations(all_articles,
+                order_by=self.settings['ARTICLE_ORDER_BY'])
         self.drafts, self.drafts_translations = \
             process_translations(all_drafts)
 
@@ -594,10 +628,13 @@ class PagesGenerator(CachingGenerator):
                         context_signal=signals.page_generator_context,
                         context_sender=self)
                 except Exception as e:
-                    logger.warning('Could not process {}\n{}'.format(f, e))
+                    logger.error('Could not process %s\n%s', f, e,
+                        exc_info=self.settings.get('DEBUG', False))
+                    self._add_failed_source_path(f)
                     continue
 
                 if not is_valid_content(page, f):
+                    self._add_failed_source_path(f)
                     continue
 
                 self.cache_data(f, page)
@@ -609,11 +646,11 @@ class PagesGenerator(CachingGenerator):
             elif page.status == "hidden":
                 hidden_pages.append(page)
             else:
-                logger.warning("Unknown status %s for file %s, skipping it." %
-                               (repr(page.status),
-                                repr(f)))
+                logger.error("Unknown status '%s' for file %s, skipping it.",
+                               page.status, f)
 
-        self.pages, self.translations = process_translations(all_pages)
+        self.pages, self.translations = process_translations(all_pages,
+                order_by=self.settings['PAGE_ORDER_BY'])
         self.hidden_pages, self.hidden_translations = (
             process_translations(hidden_pages))
 
@@ -656,7 +693,14 @@ class StaticGenerator(Generator):
     def generate_context(self):
         self.staticfiles = []
         for f in self.get_files(self.settings['STATIC_PATHS'],
+                                exclude=self.settings['STATIC_EXCLUDES'],
                                 extensions=False):
+
+            # skip content source files unless the user explicitly wants them
+            if self.settings['STATIC_EXCLUDE_SOURCES']:
+                if self._is_potential_source_path(f):
+                    continue
+
             static = self.readers.read_file(
                 base_path=self.path, path=f, content_class=Static,
                 fmt='static', context=self.context,
@@ -679,7 +723,7 @@ class StaticGenerator(Generator):
             save_as = os.path.join(self.output_path, sc.save_as)
             mkdir_p(os.path.dirname(save_as))
             shutil.copy2(source_path, save_as)
-            logger.info('copying {} to {}'.format(sc.source_path, sc.save_as))
+            logger.info('Copying %s to %s', sc.source_path, sc.save_as)
 
 
 class SourceFileGenerator(Generator):
@@ -694,7 +738,7 @@ class SourceFileGenerator(Generator):
         copy(obj.source_path, dest)
 
     def generate_output(self, writer=None):
-        logger.info(' Generating source files...')
+        logger.info('Generating source files...')
         for obj in chain(self.context['articles'], self.context['pages']):
             self._create_source(obj)
             for obj_trans in obj.translations:
