@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals, print_function
 
+import collections
 import os
 import sys
 from tempfile import mkdtemp
@@ -10,6 +11,7 @@ import logging
 import subprocess
 
 from pelican import Pelican
+from pelican.generators import StaticGenerator
 from pelican.settings import read_settings
 from pelican.tests.support import LoggedTestCase, mute, locale_available, unittest
 
@@ -75,6 +77,19 @@ class TestPelican(LoggedTestCase):
         assert not out, out
         assert not err, err
 
+    def test_order_of_generators(self):
+        # StaticGenerator must run last, so it can identify files that
+        # were skipped by the other generators, and so static files can
+        # have their output paths overridden by the {attach} link syntax.
+
+        pelican = Pelican(settings=read_settings(path=None))
+        generator_classes = pelican.get_generator_classes()
+
+        self.assertTrue(generator_classes[-1] is StaticGenerator,
+            "StaticGenerator must be the last generator, but it isn't!")
+        self.assertIsInstance(generator_classes, collections.Sequence,
+            "get_generator_classes() must return a Sequence to preserve order")
+
     def test_basic_generation_works(self):
         # when running pelican without settings, it should pick up the default
         # ones and generate correct output without raising any exception
@@ -182,5 +197,5 @@ class TestPelican(LoggedTestCase):
         logger.setLevel(orig_level)
         self.assertLogCountEqual(
             count=2,
-            msg="writing .*",
+            msg="Writing .*",
             level=logging.INFO)
