@@ -263,6 +263,9 @@ class TestUtils(LoggedTestCase):
         self.assertEqual(utils.strftime(d, '%d/%m/%y'), '29/08/12')
         self.assertEqual(utils.strftime(d, '%d/%m/%Y'), '29/08/2012')
 
+        # RFC 3339
+        self.assertEqual(utils.strftime(d, '%Y-%m-%dT%H:%M:%SZ'),'2012-08-29T00:00:00Z')
+
         # % escaped
         self.assertEqual(utils.strftime(d, '%d%%%m%%%y'), '29%08%12')
         self.assertEqual(utils.strftime(d, '%d %% %m %% %y'), '29 % 08 % 12')
@@ -280,6 +283,13 @@ class TestUtils(LoggedTestCase):
         self.assertEqual(utils.strftime(d, '%d/%m/%Y Øl trinken beim Besäufnis'),
                          '29/08/2012 Øl trinken beim Besäufnis')
 
+        # alternative formatting options
+        self.assertEqual(utils.strftime(d, '%-d/%-m/%y'), '29/8/12')
+        self.assertEqual(utils.strftime(d, '%-H:%-M:%-S'), '0:0:0')
+
+        d = utils.SafeDatetime(2012, 8, 9)
+        self.assertEqual(utils.strftime(d, '%-d/%-m/%y'), '9/8/12')
+
 
     # test the output of utils.strftime in a different locale
     # Turkish locale
@@ -461,7 +471,7 @@ class TestDateFormatter(unittest.TestCase):
                          locale_available('French'),
                          'French locale needed')
     def test_french_strftime(self):
-        # This test tries to reproduce an issue that occured with python3.3 under macos10 only
+        # This test tries to reproduce an issue that occurred with python3.3 under macos10 only
         locale.setlocale(locale.LC_ALL, str('fr_FR.UTF-8'))
         date = utils.SafeDatetime(2014,8,14)
         # we compare the lower() dates since macos10 returns "Jeudi" for %A whereas linux reports "jeudi"
