@@ -42,8 +42,11 @@ def strftime(date, date_format):
     replacing formatted output back.
     '''
 
+    c89_directives = 'aAbBcdfHIjmMpSUwWxXyYzZ%'
+    strip_zeros = lambda x: x.lstrip('0') or '0'
+
     # grab candidate format options
-    format_options = '%.'
+    format_options = '%[-]?.'
     candidates = re.findall(format_options, date_format)
 
     # replace candidates with placeholders for later % formatting
@@ -56,14 +59,28 @@ def strftime(date, date_format):
     formatted_candidates = []
     for candidate in candidates:
         # test for valid C89 directives only
-        if candidate[1] in 'aAbBcdfHIjmMpSUwWxXyYzZ%':
+        if candidate[-1] in c89_directives:
+            # check for '-' prefix
+            if len(candidate) == 3:
+                # '-' prefix
+                candidate = '%{}'.format(candidate[-1])
+                conversion = strip_zeros
+            else:
+                conversion = None
+
+            # format date
             if isinstance(date, SafeDatetime):
                 formatted = date.strftime(candidate, safe=False)
             else:
                 formatted = date.strftime(candidate)
+
             # convert Py2 result to unicode
             if not six.PY3 and enc is not None:
                 formatted = formatted.decode(enc)
+
+            # strip zeros if '-' prefix is used
+            if conversion:
+                formatted = conversion(formatted)
         else:
             formatted = candidate
         formatted_candidates.append(formatted)
@@ -293,46 +310,46 @@ def copy(source, destination):
         if not os.path.exists(dest_dir):
             os.makedirs(dest_dir)
         shutil.copy2(source_, destination_)
-        logger.info('copying %s to %s' % (source_, destination_))
+        logger.info('Copying %s to %s', source_, destination_)
     else:
-        logger.warning('skipped copy %s to %s' % (source_, destination_))
+        logger.warning('Skipped copy %s to %s', source_, destination_)
 
 
 def clean_output_dir(path, retention):
     """Remove all files from output directory except those in retention list"""
 
     if not os.path.exists(path):
-        logger.debug("Directory already removed: %s" % path)
+        logger.debug("Directory already removed: %s", path)
         return
 
     if not os.path.isdir(path):
         try:
             os.remove(path)
         except Exception as e:
-            logger.error("Unable to delete file %s; %s" % (path, str(e)))
+            logger.error("Unable to delete file %s; %s", path, e)
         return
 
     # remove existing content from output folder unless in retention list
     for filename in os.listdir(path):
         file = os.path.join(path, filename)
         if any(filename == retain for retain in retention):
-            logger.debug("Skipping deletion; %s is on retention list: %s" \
-                         % (filename, file))
+            logger.debug("Skipping deletion; %s is on retention list: %s",
+                         filename, file)
         elif os.path.isdir(file):
             try:
                 shutil.rmtree(file)
-                logger.debug("Deleted directory %s" % file)
+                logger.debug("Deleted directory %s", file)
             except Exception as e:
-                logger.error("Unable to delete directory %s; %s" % (
-                        file, str(e)))
+                logger.error("Unable to delete directory %s; %s", 
+                        file, e)
         elif os.path.isfile(file) or os.path.islink(file):
             try:
                 os.remove(file)
-                logger.debug("Deleted file/link %s" % file)
+                logger.debug("Deleted file/link %s", file)
             except Exception as e:
-                logger.error("Unable to delete file %s; %s" % (file, str(e)))
+                logger.error("Unable to delete file %s; %s", file, e)
         else:
-            logger.error("Unable to delete %s, file type unknown" % file)
+            logger.error("Unable to delete %s, file type unknown", file)
 
 
 def get_relative_path(path):
@@ -423,7 +440,7 @@ def truncate_html_words(s, num, end_text='...'):
     return out
 
 
-def process_translations(content_list):
+def process_translations(content_list, order_by=None):
     """ Finds translation and returns them.
 
     Returns a tuple with two lists (index, translations).  Index list includes
@@ -433,6 +450,14 @@ def process_translations(content_list):
     the same slug have that metadata.
 
     For each content_list item, sets the 'translations' attribute.
+
+    order_by can be a string of an attribute or sorting function. If order_by
+    is defined, content will be ordered by that attribute or sorting function.
+    By default, content is ordered by slug.
+
+    Different content types can have default order_by attributes defined
+    in settings, e.g. PAGES_ORDER_BY='sort-order', in which case `sort-order`
+    should be a defined metadata attribute in each page.
     """
     content_list.sort(key=attrgetter('slug'))
     grouped_by_slugs = groupby(content_list, attrgetter('slug'))
@@ -455,10 +480,10 @@ def process_translations(content_list):
             lang_items = list(lang_items)
             len_ = len(lang_items)
             if len_ > 1:
-                logger.warning('There are %s variants of "%s" with lang %s' \
-                        % (len_, slug, lang))
+                logger.warning('There are %s variants of "%s" with lang %s',
+                    len_, slug, lang)
                 for x in lang_items:
-                    logger.warning('    %s' % x.source_path)
+                    logger.warning('\t%s', x.source_path)
 
         # find items with default language
         default_lang_items = list(filter(attrgetter('in_default_lang'),
@@ -469,15 +494,32 @@ def process_translations(content_list):
             default_lang_items = items[:1]
 
         if not slug:
-            logger.warning((
-                    'empty slug for {!r}. '
+            logger.warning(
+                    'empty slug for %s. '
                     'You can fix this by adding a title or a slug to your '
-                    'content'
-                    ).format(default_lang_items[0].source_path))
+                    'content',
+                    default_lang_items[0].source_path)
         index.extend(default_lang_items)
         translations.extend([x for x in items if x not in default_lang_items])
         for a in items:
             a.translations = [x for x in items if x != a]
+
+    if order_by:
+        if callable(order_by):
+            try:
+                index.sort(key=order_by)
+            except Exception:
+                logger.error('Error sorting with function {}'.format(order_by))
+        elif order_by == 'basename':
+            index.sort(key=lambda x: os.path.basename(x.source_path or ''))
+        elif order_by != 'slug':
+            try:
+                index.sort(key=attrgetter(order_by))
+            except AttributeError:
+                error_msg = ('There is no "{}" attribute in the item metadata.'
+                             'Defaulting to slug order.')
+                logger.warning(error_msg.format(order_by))
+
     return index, translations
 
 
@@ -499,7 +541,7 @@ def folder_watcher(path, extensions, ignores=[]):
                     try:
                         yield os.stat(os.path.join(root, f)).st_mtime
                     except OSError as e:
-                        logger.warning('Caught Exception: {}'.format(e))
+                        logger.warning('Caught Exception: %s', e)
 
     LAST_MTIME = 0
     while True:
@@ -522,7 +564,7 @@ def file_watcher(path):
             try:
                 mtime = os.stat(path).st_mtime
             except OSError as e:
-                logger.warning('Caught Exception: {}'.format(e))
+                logger.warning('Caught Exception: %s', e)
                 continue
 
             if mtime > LAST_MTIME:
@@ -538,7 +580,9 @@ def set_date_tzinfo(d, tz_name=None):
     """Set the timezone for dates that don't have tzinfo"""
     if tz_name and not d.tzinfo:
         tz = pytz.timezone(tz_name)
-        return tz.localize(d)
+        d = tz.localize(d)
+        return SafeDatetime(d.year, d.month, d.day, d.hour, d.minute, d.second,
+                            d.microsecond, d.tzinfo)
     return d
 
 
@@ -596,15 +640,15 @@ class FileDataCacher(object):
                 with self._cache_open(self._cache_path, 'rb') as fhandle:
                     self._cache = pickle.load(fhandle)
             except (IOError, OSError) as err:
-                logger.debug(('Cannot load cache {} (this is normal on first '
-                    'run). Proceeding with empty cache.\n{}').format(
-                        self._cache_path, err))
+                logger.debug('Cannot load cache %s (this is normal on first '
+                    'run). Proceeding with empty cache.\n%s',
+                        self._cache_path, err)
                 self._cache = {}
             except Exception as err:
-                logger.warning(('Cannot unpickle cache {}, cache may be using '
+                logger.warning(('Cannot unpickle cache %s, cache may be using '
                     'an incompatible protocol (see pelican caching docs). '
-                    'Proceeding with empty cache.\n{}').format(
-                        self._cache_path, err))
+                    'Proceeding with empty cache.\n%s'),
+                        self._cache_path, err)
                 self._cache = {}
         else:
             self._cache = {}
@@ -629,15 +673,15 @@ class FileDataCacher(object):
                 with self._cache_open(self._cache_path, 'wb') as fhandle:
                     pickle.dump(self._cache, fhandle)
             except (IOError, OSError, pickle.PicklingError) as err:
-                logger.warning('Could not save cache {}\n{}'.format(
-                    self._cache_path, err))
+                logger.warning('Could not save cache %s\n ... %s',
+                    self._cache_path, err)
 
 
 class FileStampDataCacher(FileDataCacher):
     '''Subclass that also caches the stamp of the file'''
 
     def __init__(self, settings, cache_name, caching_policy, load_policy):
-        '''This sublcass additionaly sets filestamp function
+        '''This sublcass additionally sets filestamp function
         and base path for filestamping operations
         '''
         super(FileStampDataCacher, self).__init__(settings, cache_name,
@@ -656,8 +700,7 @@ class FileStampDataCacher(FileDataCacher):
                         return hash_func(fhandle.read()).digest()
                 self._filestamp_func = filestamp_func
             except AttributeError as err:
-                logger.warning('Could not get hashing function\n{}'.format(
-                    err))
+                logger.warning('Could not get hashing function\n\t%s', err)
                 self._filestamp_func = None
 
     def cache_data(self, filename, data):
@@ -677,8 +720,8 @@ class FileStampDataCacher(FileDataCacher):
         try:
             return self._filestamp_func(filename)
         except (IOError, OSError, TypeError) as err:
-            logger.warning('Cannot get modification stamp for {}\n{}'.format(
-                filename, err))
+            logger.warning('Cannot get modification stamp for %s\n\t%s',
+                filename, err)
             return b''
 
     def get_cached_data(self, filename, default=None):
